
import math 
import ufl
from dolfin import *

parameters["form_compiler"]["name"]= "sfc"

class Source(Expression): 
    def eval(self, values, x): 
        values[0] = -2 + math.exp(x[0]*x[0]) 

class Solution(Expression): 
    def eval(self, values, x): 
        values[0] = x[0]*x[0] 

class DirichletBoundary(SubDomain): 
    def inside(self, x, on_boundary): 
        return bool(on_boundary)

# Geometry
N = 80  
mesh = UnitSquare(N,N)

# Function spaces
X = FunctionSpace(mesh, 'CG', 1)

# Unknown function 
U = Function(X)

# Basis functions 
v = TestFunction(X)
u = TrialFunction(X)

# Source term 
source = Source(element=X.ufl_element())
s = Function(X)
s.interpolate(source)

# Forms
F = inner(grad(U),grad(v))*dx + exp(U)*v*dx  
J = derivative(F, U, u)

# Analytical solution 
solution = Solution(element=X.ufl_element())
sol = project(solution, X)

# Solve nonlinear variational problem
dirichlet_boundary = DirichletBoundary()
bc = DirichletBC(X, sol, dirichlet_boundary)

solve(F == 0, U, bc, J=J)

# Write solution to file
file = File("u.pvd") 
file << U 

file = File("sol.pvd") 
file << sol 

# Plot and wait
plot(sol)
plot(U)
interactive()
